from enum import Enum


class Theme(str, Enum):
    LIGHT = "light"
    DARK = "dark"
    CUPCAKE = "cupcake"
    BUMBLEBEE = "bumblebee"
    EMERALD = "emerald"
    CORPORATE = "corporate"
    SYNTHWAVE = "synthwave"
    RETRO = "retro"
    CYBERPUNK = "cyberpunk"
    VALENTINE = "valentine"
    HALLOWEEN = "halloween"
    GARDEN = "garden"
    FOREST = "forest"
    AQUA = "aqua"
    LOFI = "lofi"
    PASTEL = "pastel"
    FANTASY = "fantasy"
    WIREFRAME = "wireframe"
    BLACK = "black"
    LUXURY = "luxury"
    DRACULA = "dracula"
    CMYK = "cmyk"
    AUTUMN = "autumn"
    BUSINESS = "business"
    ACID = "acid"
    LEMONADE = "lemonade"
    NIGHT = "night"
    COFFEE = "coffee"
    WINTER = "winter"
    DIM = "dim"
    NORD = "nord"
    SUNSET = "sunset"
